﻿using MathNet.Numerics.Statistics;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ConsoleApp1
{
    public class Program
    {
        static void Main(string[] args)
        {
            List<double> dblHeights = LoadData("data.txt");

            Console.WriteLine($"# of data: {dblHeights.Count}");

            double mean = Statistics.Mean(dblHeights);
            Console.WriteLine($"Mean: {mean}");

            Console.WriteLine();
            {
                Console.WriteLine($"MathNet - Variance: {Statistics.Variance(dblHeights)}");
                Console.WriteLine($"MathNet - Standard Deviation: {Statistics.StandardDeviation(dblHeights)}");
            }

            Console.WriteLine();

            Console.WriteLine($"Variance: {Variance(dblHeights.ToArray(), mean, false)}");
            Console.WriteLine($"Variance with Bessel's correction: {Variance(dblHeights.ToArray(), mean, true)}");

            Console.WriteLine();

            {
                Console.WriteLine($"MathNet - Variance: {Statistics.PopulationVariance(dblHeights)}");
                Console.WriteLine($"MathNet - Standard Deviation: {Statistics.PopulationStandardDeviation(dblHeights)}");
            }
        }

        public static double Variance(double[] samples, double mean, bool useBesselCorrection)
        {
            if (samples.Length <= ((useBesselCorrection == true) ? 1 : 0))
            {
                return double.NaN;
            }

            double sum = 0;

            for (int i = 0; i < samples.Length; i++)
            {
                double diff = samples[i] - mean;
                sum += (diff * diff);
            }

            double variance = sum / ((samples.Length - ((useBesselCorrection == true) ? 1 : 0)));
            return variance;
        }

        /*
        public static double MathNet_Variance(double[] samples)
        {
            if (samples.Length <= 1)
            {
                return double.NaN;
            }

            double num = 0.0;
            double num2 = samples[0];

            for (int i = 1; i < samples.Length; i++)
            {
                num2 += samples[i];
                double difference = ((i + 1) * samples[i]) - num2;
                num += (difference * difference) / ((i + 1.0) * i);
            }

            return (num / ((double)(samples.Length - 1)));
        }
        */

        public static List<double> LoadData(string fileName)
        {
            List<double> list = new List<double>();

            foreach (string txt in File.ReadAllLines(fileName))
            {
                if (double.TryParse(txt, out double result) == true)
                {
                    list.Add(result);
                }
            }

            return list;
        }
    }
}
